﻿//css_ref System.Data;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.Attributes;

using System;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Attributes;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Lino;


namespace AddUserFieldsAndUserQueries
{
    class Program
    {
        //All most all user defined fields(UDF) can be added manual except these 2. Therefor this script can be used for doing that

        const string CONNECTION_STRING = "Server=localhost;CompanyDB=SBODemo_US;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=manager;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2005;DBCultureInfo=en-US;LicenseServer=localhost:30000;ExtraDb=PMX_SBODemo_US";


        [SboColumnValue("Y", "Yes")]
        [SboColumnValue("N", "No")]
        [SboColumnValue("C", "Customer Label")]
        public class PmxPrintItemLabelConvertor : ISboBOPropertyColumnConvertor
        {

            #region ISboBoPropertyColumnConverter Members

            /// <summary>
            /// Convert column value to the property of the business object
            /// </summary>
            /// <param name="columnValue">The column value</param>
            /// <returns>The business object property value</returns>
            public object ConvertColumnToBOProperty(object columnValue)
            {
                return null;
            }

            /// <summary>
            /// Convert business object value to database column value
            /// </summary>
            /// <param name="propertyValue">The business object property value</param>
            /// <returns>The database column value</returns>
            public object ConvertBOPropertyToColumn(object propertyValue)
            {
                return null;
            }

            #endregion
        }

        static void Main(string[] args)
        {
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

                    SboUserFieldMDProvider udfProv = new SboUserFieldMDProvider(conn);

                    //Add item label report layout
                    udfProv.AddUpdateUserfield("OCRD", new SboColumnDefinitionAttribute("PMX_LBL_REPORT", "Label Report", SboFieldType.Numeric));

                    //Add card item sales price
                    udfProv.AddUpdateUserfield("OSCN", new SboColumnDefinitionAttribute("PMX_SAL_PRICE", "Sales Price", SboFieldType.Float, 11 , SboFieldSubType.Price));
                    //Add card item barcode
                    udfProv.AddUpdateUserfield("OSCN", new SboColumnDefinitionAttribute("PMX_ITM_CODEBARS", "Item barcode", SboFieldType.Alpha, 254, SboFieldSubType.None));
                    //Add card item description
                    udfProv.AddUpdateUserfield("OSCN", new SboColumnDefinitionAttribute("PMX_ITM_DESCR", "Item description", SboFieldType.Alpha, 254, SboFieldSubType.None));


                    //Add print item label
                    SboColumnDefinitionAttribute columnDef = new SboColumnDefinitionAttribute("PMX_PR_ITM_LBL", "Print item label?", SboFieldType.Alpha, 1, SboFieldSubType.None);
                    columnDef.SboBOColumnConverterType = typeof(PmxPrintItemLabelConvertor);
                    columnDef.DefaultValue = "N";
                    //udfProv.AddUpdateUserfield("OCRD", columnDef);
                    udfProv.AddUpdateUserfield("RDR1", columnDef);


                    ////add production flow
                    //AddUpdateWindowsWorkflow(PmxWorkflowExecutionTypes.SUB_FLOW, "ProductionFlow",
                    //    "Produmex.Sbo.Logex.Wwf.Workflows.ProductionFlow, Produmex.Sbo.Logex.Wwf.Workflows", conn);


                    //AddUpdateExtension("LINO_PDN", "Lino Purchase delivery generator", 
                    //    typeof(Produmex.Sbo.Logex.Lino.Generators.PurchaseDeliveryGenerators.PurchaseDeliveryGenerator), 
                    //    Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IPURCHASEDELIVERYGENERATOR,
                    //    conn);


                }

                //Complete transaction
                scope.Complete();

                Console.WriteLine("Done ...");
            }
        }

        /// <summary>
        /// Adds the update workflow.
        /// </summary>
        /// <param name="executionType">Type of the execution.</param>
        /// <param name="name">The name.</param>
        /// <param name="workflowType">Type of the workflow.</param>
        /// <param name="dbConn">The db conn.</param>
        private static void AddUpdateWindowsWorkflow(string executionType, string name, string workflowType, SboConnection dbConn)
        {
            PmxWorkflow workflow = null;
            try
            {
                PmxWorkflowProvider wfProvider = new PmxWorkflowProvider(dbConn);
                workflow = wfProvider.GetWorkflowByTypeName(workflowType, null);

                PmxWorkflowWwfProvider wwfProvider = new PmxWorkflowWwfProvider(dbConn);
                PmxWorkflowWwf wwfWorkflow = ((workflow == null) ? null : wfProvider.GetSpecializedWorkflow(workflow.InternalKey)) as PmxWorkflowWwf;

                bool newType;
                if (wwfWorkflow == null)
                {
                    wwfWorkflow = wwfProvider.GetNewBO();
                    newType = true;
                }
                else
                {
                    newType = false;
                }

                // Set values
                wwfWorkflow.Name = name;
                wwfWorkflow.TypeName = workflowType;
                wwfWorkflow.ExecutionType = executionType;

                //Add/update type
                if (newType)
                {
                    wwfProvider.AddBO(wwfWorkflow);
                }
                else
                {
                    wwfProvider.UpdateBO(wwfWorkflow, false, false);
                }

            }
            finally
            {
                if (workflow != null)
                {
                    workflow.Dispose();
                    workflow = null;
                }
            }

        }

        /// <summary>
        /// Add/Update <see cref="PmxExtension"/>
        /// </summary>
        /// <param name="code">The code of the extension.</param>
        /// <param name="name">The name of the extension.</param>
        /// <param name="type">The type of the extension.</param>
        /// <param name="extensionTypeCode">The extension type code.</param>
        /// <param name="dbConn">The db conn.</param>
        private static void AddUpdateExtension(string code, string name, Type type, string extensionTypeCode, SboConnection dbConn)
        {
            PmxExtensionProvider prov = new PmxExtensionProvider(dbConn);
            PmxExtension extension = null;
            try
            {
                bool newBO = false;
                extension = prov.GetBO(code);
                if (extension == null)
                {
                    newBO = true;
                    extension = prov.GetNewBO();
                    extension.Code = code;
                }
                extension.Name = name;
                extension.ExtensionTypeCode = extensionTypeCode;
                extension.Type = Produmex.Foundation.Reflection.ReflectionHelper.GetTypeAndAssemblyName(type);
                if (newBO)
                {
                    prov.AddBO(extension);
                }
                else
                {
                    prov.UpdateBO(extension, false, false);
                }
            }
            finally
            {
                if (extension != null) extension.Dispose();
            }
        }
    }
}
