﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Threading;

using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;

using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation;

namespace WorkflowScript_LinoMainProduction
{
    public class WorkflowScript_LinoMainProduction : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Private fields
        private CultureInfo m_defaultCultureInfo;
        private PmxOseCompany m_pmxOseCompany;

        // Input parameters
        // <none>

        // Output parameters
        // <none>

        // Sub-flows

        private WorkflowInstanceBase m_linoProduction;
        private WorkflowInstanceBase m_getInitialData;

        #region DEFINITIONS

        private static class Definitions
        {
            public const string UserName = "PMX_TC";

            public const string Password = "produmex";
        }

        #endregion


        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_LinoMainTouchProduction"/> class.
        /// </summary>
        /// <param name="parent">The parent flow instance.</param>
        /// <param name="factory">The workflow factory.</param>
        public WorkflowScript_LinoMainProduction(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {

            // Initialize required sub-flows
            m_getInitialData = Create("GetInitialDataScript");
            m_getInitialData.BindParameters(
                new ReadWriteBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                new ReadWriteBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany")
            );


            m_linoProduction = Create("LinoProduction");
            m_linoProduction.BindParameters(
                new ReadOnlyBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
                new ReadOnlyBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany")
            );
        }

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;

            //// Execute sub-flow
            //PassMessagesOnTo(m_getInitialData);
            //m_getInitialData.StartAndWait();
            //PassMessagesOnTo(null);

            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            sboProviderService.LogOff();
            //LogOn
            ProdumexError pmxError = sboProviderService.LogOn(
                Definitions.UserName,
                Definitions.Password);

            if (pmxError != null)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    m_defaultCultureInfo,
                    BuildParamCollection(
                        "PmxError", pmxError,
                        "ShowButton", true));
                WaitForMessage();

                return;
            }

            sboProviderService.InvokeMethodWithDbConnection<bool>(false, null, null,
            new DelegateWithSboConnection<bool>(ExecuteLogon),
            Definitions.UserName);


            base.StartSubFlowAndWait(m_linoProduction, true);


           


        }

        private bool ExecuteLogon(SboConnection conn, object[] parameters)
        {
            string userName = (string)parameters[0];

            Produmex.Sbo.Logex.Data.Providers.PmxUserProvider userProv = new Produmex.Sbo.Logex.Data.Providers.PmxUserProvider(conn);
            Produmex.Sbo.Logex.Data.BusinessObjects.PmxUser pmxUser = userProv.GetUserByCode(userName);

            if (pmxUser != null && pmxUser.UserLanguageKey != null)
            {
                Produmex.Foundation.Data.Sbo.Providers.PmxUserLanguageProvider prov = new Produmex.Foundation.Data.Sbo.Providers.PmxUserLanguageProvider(conn);
                m_defaultCultureInfo = prov.GetCultureInfo((int)pmxUser.UserLanguageKey);
            }

            return true;
        }
    }

}
