﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;

using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;

using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.DbClient;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_LinoProduction
{
	/// <summary>
	/// Represents the stucture of the extended sbo business partner table.
	/// </summary>
    public static class PmxBusinessPartnerTableLino
    {
        /// <summary>
        /// The table name of the business partner
        /// </summary>
        public const string NAME = SboBusinessPartnerTable.NAME;

        /// <summary>
        /// Represents the definition of all columns in this table
        /// </summary>
        public static class Columns
        {
            /// <summary>
            /// The ItemLabelReport column
            /// </summary>
            public static class ItemLabelReport
            {
                /// <summary>
                /// The name of the column
                /// </summary>
                public const string NAME = "U_PMX_LBL_REPORT";

            }

        }
    }

    /// <summary>
    /// Represents the stucture of the extended sbo sales order line table.
    /// </summary>
    public static class PmxSalesOrderLineTableLino
    {
        /// <summary>
        /// The table name of the sales order line
        /// </summary>
        public const string NAME = SboSalesOrderLineTable.NAME;

        /// <summary>
        /// Represents the definition of all columns in this table
        /// </summary>
        public static class Columns
        {

            /// <summary>
            /// The PrintItemLabel column
            /// </summary>
            public static class PrintItemLabel
            {
                /// <summary>
                /// The name of the column
                /// </summary>
                public const string NAME = "U_PMX_PR_ITM_LBL";
            }


        }
    }

    public class WorkflowScript_LinoProduction : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        private static class Definitions
        {
            //report definitions
            public const string REPORT_PATH_PICKLIST_TASKS = @"LinoPickListTasks.rpt";
            public const string REPORT_PATH_PALLET_LABEL = @"LinoPalletLabel.rpt";

            public const string OUTPUT_LOCATIONCODE = "BIN01";

            //U_PMX_PR_LBL
            public const string PRINT_LABEL = "Y";
            public const string DO_NOT_PRINT_LABEL = "N";
            public const string CUSTOMER_LABEL = "C";

            public const int EXTRA_LABELS = 5;

            public const int NUMBER_OF_PALLET_LABELS = 0;


            //column names
            public const string COLUMN_NAME_SALES_ITEM_ITEM_CODE = "SalesItemItemCode";
            public const string COLUMN_NAME_SALES_ITEM_QUANTITY = "SalesItemQuantity";
            public const string COLUMN_NAME_PART_QUANTITY = "PartQuantity";
        }



        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;

        // Input parameters
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        // Output parameters
        // <none>


        private CultureInfo m_defaultCultureInfo;
        private PmxOseCompany m_pmxOseCompany;


        public WorkflowScript_LinoProduction(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }


        #region EXECUTE


        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            string packingLineCode = null;
            string warehouseCode = null;
            string locationCode = null;

            string initialErrorKey = null;
            string deviceID = session.DeviceInstanceId;
            int? pickListDocEntry;

            string sscc = null;
            string cardCode = null;
            string cardName = null;
            string shipToAddress = null;
            int? luid = null;
            string pickPackRemarks = null;

            string buttonKeySelectPackingTask = null;
            DataSet dsPickList = null;

            int? userSign = null;
            int numberOfLabelsToPrint = 0;
            DocumentRef[] listOfSalesDeliveryDocEntries = null;

            m_defaultCultureInfo = DefaultCultureInfo.Get();
            m_pmxOseCompany = PmxOseCompany.Get();

            //We do standard production
            WorkflowInstanceBase m_productionFlow = Create("ProductionFlow");
            m_productionFlow.BindParameters(
            new ReadOnlyBinder<CultureInfo>("DefaultCultureInfo", this, "m_defaultCultureInfo"),
            new ReadOnlyBinder<PmxOseCompany>("PmxOseCompany", this, "m_pmxOseCompany"));
            ProdumexError PmxError = null;

            Dictionary<string, PmxItemInfo> itemInfos = new Dictionary<string,PmxItemInfo>();



        Step_SelectTask:

            #region SELECT TASK

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_TASK",
                        "AllowedButtons", new string[] { "MSG_BUTTON_PRODUCTION_LINO", "MSG_BUTTON_PACKING_LINO" },
                        "ShowStopButton", true));
            Message msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                string buttonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKey == "MSG_BUTTON_PRODUCTION_LINO")
                {
                    // Execute sub-flow
                    base.StartSubFlowAndWait(m_productionFlow, true);
                  


                    goto Step_SelectTask;
                }
                else
                {
                    //check if we have a production line above the device
                    locationCode = sboProviderService.InvokeMethodWithDbConnection<string>(false, false, null, null, new DelegateWithPmxDbConnection<string>(ExecuteGetParentProductionLineForLocation), deviceID);
                    goto Step_SelectPackagingLine;
                }
            }

            return;

            #endregion

        Step_SelectPackagingLine:

            #region SELECT PACKAGING LINE

            //get the list of packaging lines
            string query = BuildQuery.GetPackingLines(null, locationCode);
            DataSet dsPackingLines = sboProviderService.RunView(false, null, null, query);


            packingLineCode = null;
            warehouseCode = null;


            if (this.GetNumberOfRows(dsPackingLines) == 0)
            {
                initialErrorKey = "MSG_ERROR_NO_PACKAGING_LINE_FOUND";
                goto Step_SelectTask;
            }

            if (this.GetNumberOfRows(dsPackingLines) == 1 &&
                !string.IsNullOrEmpty(locationCode))
            {
                //Only one production line, and there is a location found for the device
                packingLineCode = locationCode;
            }
            else
            {
                //Let user select location
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_PACKING_LINE",
                        "LocationDS", dsPackingLines
                        ));
                msg = WaitForMessage();

                //reset data
                initialErrorKey = null;

                locationCode = null;


                if (msg.Name.EndsWith(".LocationSelected"))
                {
                    packingLineCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                }
                else
                {
                    goto Step_SelectTask;
                }
            }



            //get the other locations
            DataRow[] rows = dsPackingLines.Tables[0].Select(LocationDataSet.Columns.CODE.Name + " = '" + packingLineCode + "'");
            if (rows.Length > 0)
            {
                DataRow row = rows[0];
                warehouseCode = row[LocationDataSet.Columns.WAREHOUSECODE.Name] as string;
            }

            goto Step_SelectPackingTask;

            #endregion

        Step_SelectPackingTask:

            #region SELECT PACKING TASK

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IScanStringValueScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SCAN_PICKLIST_NUMBER",
                        "AllowedButtonKeys", new string[] { "MSG_BUTTON_START_PICKLIST", "MSG_BUTTON_PRINT_PALLET_LABEL", "MSG_BUTTON_STOP_PICKLIST", "MSG_BUTTON_PRINT_ITEM_LABEL", "MSG_BUTTON_PACK_MORE_LINO"},
                        "ForceDataEntry", true,
                        "AllowToGoBack", true,
                        "DefaultValue", "--> BarCode <--"));
            msg = WaitForMessage();

            initialErrorKey = null;

            sscc = null;
            cardCode = null;
            cardName = null;
            shipToAddress = null;
            luid = null;
            pickPackRemarks = null;
            buttonKeySelectPackingTask = null;
            listOfSalesDeliveryDocEntries = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeySelectPackingTask = ExtractParameter<string>(msg.Parameters, "buttonKey");

                switch (buttonKeySelectPackingTask)
                {
                    case "MSG_BUTTON_START_PICKLIST":
                        goto Step_SelectPickList;
                    case "MSG_BUTTON_PRINT_PALLET_LABEL":
                        goto Step_SelectPickList;
                    case "MSG_BUTTON_PRINT_ITEM_LABEL":
                        goto Step_PrintManualItemLabel;
                    case "MSG_BUTTON_STOP_PICKLIST":
                        goto Step_SelectPickList;
                    case "MSG_BUTTON_PACK_MORE_LINO":
                        goto Step_SelectPickList;
                    default: break;
                }

            }

            if (msg.Name.EndsWith(".StringScanned"))
            {
                string pickListNumber = ExtractParameter<string>(msg.Parameters, "stringValue");


                int tempPicklistDocEntry;
                if (int.TryParse(pickListNumber, out tempPicklistDocEntry))
                {

                    //does the picklist exist?
                    query = BuildQuery.GetPickListData(tempPicklistDocEntry);
                    dsPickList = sboProviderService.RunView(false, null, null, query);

                    if (this.GetNumberOfRows(dsPickList) > 0)
                    {
                        //fill the data from the first row
                        pickListDocEntry = tempPicklistDocEntry;

                        DataRow row = dsPickList.Tables[0].Rows[0];

                        cardCode = row[PmxPickListTable.Columns.CardCode.NAME] as string;
                        cardName = row[PmxPickListTable.Columns.CardName.NAME] as string;
                        shipToAddress = row[PmxPickListTable.Columns.ShipToAddress.NAME] as string;
                        pickPackRemarks = row[PmxPickListTable.Columns.PickPackRemarks.NAME] as string;
                        sscc = row[PmxLogisticUnitIDTable.Columns.SSCC.NAME] as string;
                        int tempLUID;
                        if (int.TryParse(Convert.ToString(row[PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME]), out tempLUID))
                        {
                            luid = tempLUID;
                        }
                        else
                        {
                            luid = null;
                        }

                        goto Step_PackItems;
                    }
                }

                //no picklist found
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                       "InitialErrorKey", initialErrorKey,
                       "MessageKey", "MSG_ERROR_NO_PICKLIST_FOUND",
                       "MessageAddendum", pickListNumber,
                       "ShowButton", true));

                //msg = WaitForMessage();

                goto Step_SelectPackingTask;
            }

            goto Step_SelectTask;


            #endregion

            #region START PICKLIST

        Step_SelectPickList:

            #region SELECT PICK LIST

            pickListDocEntry = null;
            query = BuildQuery.GetPickListForPacking(buttonKeySelectPackingTask == "MSG_BUTTON_STOP_PICKLIST" ||
                                                    buttonKeySelectPackingTask == "MSG_BUTTON_PRINT_PALLET_LABEL" ||
                                                    buttonKeySelectPackingTask == "MSG_BUTTON_PACK_MORE_LINO");

            DataSet dsPickLists = sboProviderService.RunView(false, null, null, query); ;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.ISelectPickListScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_PICKLIST",
                "PickListDS", dsPickLists
                ));
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".PickListSelected"))
            {
                pickListDocEntry = ExtractParameter<int>(msg.Parameters, "pickListDocEntry");


                switch (buttonKeySelectPackingTask)
                {
                    case "MSG_BUTTON_START_PICKLIST":
                        goto Step_PrintTaskReport;
                    case "MSG_BUTTON_PRINT_PALLET_LABEL":
                        goto Step_PrintPalletLabel;
                    case "MSG_BUTTON_STOP_PICKLIST":
                        goto Step_StopPickList;
                    case "MSG_BUTTON_PACK_MORE_LINO":
                        goto Step_PackMore;
                    default: break;
                }

            }

            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectPickList;
            }

            goto Step_SelectPackingTask;

            #endregion

        Step_PrintTaskReport:

            #region PRINT TASK REPORT

            //Show screen to inform user we are printing documents
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
                   "InitialErrorKey", initialErrorKey,
                   "MessageKey", "MSG_GETTING_REPORTS_TO_PRINT",
                   "ShowButton", false));


            //we need to print the report
            try
            {
                //tasks
                sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintPickListTasks), pickListDocEntry, deviceID, packingLineCode, false);
		
		if(Definitions.NUMBER_OF_PALLET_LABELS > 0)
		{
                    //pallet label
                    sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintPalletLabel), pickListDocEntry, deviceID, packingLineCode, false, Definitions.NUMBER_OF_PALLET_LABELS);
		}


                goto Step_PrintItemLabels;
            }
            catch (Produmex.Foundation.ProdumexException ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", ex.Error,
                        "ShowButton", true));

                msg = WaitForMessage();

            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                        "ShowButton", true));

                msg = WaitForMessage();
            }

            #endregion

        Step_PrintItemLabels:

            #region PRINT ITEM LABELS

            query = BuildQuery.GetItemsToPrint(pickListDocEntry.Value);
            DataSet dsListOfItemsToPrint = sboProviderService.RunView(false, null, null, query);


            int? reportKey = null;
            cardCode = null;

            Dictionary<string, double> listOfLabels = new Dictionary<string, double>();
            if (this.GetNumberOfRows(dsListOfItemsToPrint) > 0)
            {
                foreach (DataRow row in dsListOfItemsToPrint.Tables[0].Rows)
                {
		    if (Convert.ToString(row[PmxSalesOrderLineTableLino.Columns.PrintItemLabel.NAME]) == SboBooleanColumnValues.Y.VALUE)
                    {
                        string itemCode = row[ PmxPickListLineTable.Columns.ItemCode.NAME ] as string;
                        double qtyPicked = Convert.ToDouble( row[ PmxPickListLineTable.Columns.QtyPicked.NAME ] );
                        string salesItemCode = row[ Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE ] as string;
                        if ( !string.IsNullOrEmpty( salesItemCode ) )
                        {
                            // Get qtyPart and make sure it is at least 0.001 ( 1000 SalesItems per InvItem )
                            double qtyPart = Math.Max( 0.001, Convert.ToDouble( row[ Definitions.COLUMN_NAME_PART_QUANTITY ] ) );
                            if ( !listOfLabels.ContainsKey( salesItemCode ) )
                            {
                                listOfLabels.Add( salesItemCode, qtyPicked / qtyPart );
                            }
                            else
                            {
                                listOfLabels[ salesItemCode ] += qtyPicked / qtyPart;
                            }
                        }
                        else
                        {
                            if ( !listOfLabels.ContainsKey( itemCode ) )
                            {
                                listOfLabels.Add( itemCode, qtyPicked );
                            }
                            else
                            {
                                listOfLabels[ itemCode ] += qtyPicked;
                            }
                        }
		    }

                    int tempReportKey;
                    //all the items have the same report and cardcode!
                    if (int.TryParse(Convert.ToString( row[PmxBusinessPartnerTableLino.Columns.ItemLabelReport.NAME]), out tempReportKey))
                    {
                        reportKey = tempReportKey;
                    }
                    cardCode = row[PmxBusinessPartnerTable.Columns.CardCode.NAME] as string;

                }

                if (reportKey.HasValue)
                {

                    bool isFirst = true;
                    foreach (string itemCode in listOfLabels.Keys)
                    {
                        //print the list of items
                        int numberOfCopies = Convert.ToInt32( Math.Ceiling( listOfLabels[ itemCode ] ) );
                        if ( numberOfCopies > 0 )
                        {
                            numberOfCopies += Definitions.EXTRA_LABELS;

                            //we need to print the report
                            try
                            {
                                if (!isFirst)
                                {
                                    //print separate file
                                    sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintItemLabels), null, null, deviceID, packingLineCode, 1, reportKey, false);

                                }

                                sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintItemLabels), itemCode, cardCode, deviceID, packingLineCode, numberOfCopies, reportKey, false);
                            }
                            catch (Produmex.Foundation.ProdumexException ex)
                            {
                                s_log.Error(ex.ToString());
                                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                    DefaultCultureInfo.Get(), BuildParamCollection(
                                        "InitialErrorKey", initialErrorKey,
                                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                        "MessageAddendum", ex.Message,
                                        "PmxError", ex.Error,
                                        "ShowButton", true));

                                msg = WaitForMessage();
                            }
                            catch (Exception ex)
                            {
                                s_log.Error(ex.ToString());
                                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                    DefaultCultureInfo.Get(), BuildParamCollection(
                                        "InitialErrorKey", initialErrorKey,
                                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                        "MessageAddendum", ex.Message,
                                        "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                        "ShowButton", true));

                                msg = WaitForMessage();
                            }

                            isFirst = false;
                        }


                    }
                }
            }



            #endregion



            goto Step_SelectPackingTask;

            #endregion

        Step_PackItems:

            #region CONSUME PICKED ITEMS
            userSign = null;

            //get the list of items to pack
            query = BuildQuery.GetItemsToPack(pickListDocEntry.Value, true);
            DataSet dsListOfItemsToPack = sboProviderService.RunView(false, null, null, query);

            //get the list of items to pack
            query = BuildQuery.GetPackingData(pickListDocEntry.Value, packingLineCode, false, sboProviderService.GetDbTool());
            DataSet dsPackingData = sboProviderService.RunView(false, null, null, query);

            try
            {


                //build list of 'normal' items, and items that need to be assembled
                Dictionary<string, Dictionary<string, double>> listOfItems = new Dictionary<string, Dictionary<string, double>>();

                foreach (DataRow row in dsListOfItemsToPack.Tables[0].Rows)
                {
                    string itemCode = row[PmxPickListLineTable.Columns.ItemCode.NAME] as string;
                    string salesItemItemCode = row[Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE] as string;

                    if (!string.IsNullOrEmpty(salesItemItemCode))
                    {
                        if (!listOfItems.ContainsKey(salesItemItemCode))
                        {
                            listOfItems.Add(salesItemItemCode, new Dictionary<string, double>());
                        }
                        //the item is part of a sales item, so we need to set the ingredients
                        if (!listOfItems[salesItemItemCode].ContainsKey(itemCode))
                        {
                            //set the quantity of the BOM
                            double qtyOnSalesItem = Convert.ToDouble(row[SboSalesOrderLineTable.Columns.Quantity.NAME]);
                            double componentQty = Convert.ToDouble(row[PmxComponentSalesItemViewDefinition.Columns.ComponentQuantity.NAME]);
                            listOfItems[salesItemItemCode].Add(itemCode, componentQty / qtyOnSalesItem);
                        }
                    }
                    else
                    {
                        if (!listOfItems.ContainsKey(itemCode))
                        {
                            listOfItems.Add(itemCode, new Dictionary<string, double>());
                        }
                    }
                }

                Dictionary<string, double> listOfUsedIngredients = new Dictionary<string, double>();

                //we need to make the new dataset to pass to the screen
                DataSet dsCurrentPackingData = dsPackingData.Clone();
                if (this.GetNumberOfRows(dsPackingData) > 0)
                {
                    foreach (string itemCode in listOfItems.Keys)
                    {
                        if (listOfItems[itemCode].Count == 0)
                        {
                            //The item is not a sales item, so we can copy the data
                            foreach (DataRow row in dsPackingData.Tables[0].Select(PackingDataCustomDataSet.Columns.ITEMCODE.Name + " = '" + itemCode + "'"))
                            {
                                //copy the items into the new ds
                                dsCurrentPackingData.Tables[0].Rows.Add(row.ItemArray);
                            }
                        }
                        else
                        {
                            //the item is a sales item, so we need to add the sales item instead of the component
                            //calculate the max qty that can be packed
                            double maxQty = double.MaxValue;

                            DataRow[] rowsNumberOfDecimals = dsListOfItemsToPack.Tables[0].Select(Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE + " = '" + itemCode + "'");
                            int numberOfDecimals = 6;
                            if (rowsNumberOfDecimals.Length > 0)
                            {
                                numberOfDecimals = Convert.ToInt32(rowsNumberOfDecimals[0][PmxItemTable.Columns.UomDecimals.NAME]);
                            }

                            foreach (string itemCodeIngredient in listOfItems[itemCode].Keys)
                            {
                                double qtyIngredient = 0;
                                foreach (DataRow row in dsPackingData.Tables[0].Select(PackingDataCustomDataSet.Columns.ITEMCODE.Name + " = '" + itemCodeIngredient + "'"))
                                {
                                    //get the qty
                                    qtyIngredient += Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY.Name]);

                                    //only keep lowest possible number of decimal
                                    int tempDecimals = Convert.ToInt32(row[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name]);
                                    if (tempDecimals < numberOfDecimals)
                                    {
                                        numberOfDecimals = tempDecimals;
                                    }

                                }
                                //reduce the qty of the ingredient with the quantity already used
                                if (listOfUsedIngredients.ContainsKey(itemCodeIngredient))
                                {
                                    qtyIngredient -= listOfUsedIngredients[itemCodeIngredient];
                                }

                                maxQty = Math.Min(maxQty, qtyIngredient / listOfItems[itemCode][itemCodeIngredient]);


                            }

                            //we need to add all the quantities we consume of the ingredient to the list
                            foreach (string itemCodeIngredient in listOfItems[itemCode].Keys)
                            {
                                if (!listOfUsedIngredients.ContainsKey(itemCodeIngredient))
                                {
                                    listOfUsedIngredients.Add(itemCodeIngredient, 0);
                                }
                                listOfUsedIngredients[itemCodeIngredient] += (listOfItems[itemCode][itemCodeIngredient] * maxQty);
                            }

                            if (maxQty > 0)
                            {
                                //we can add a row for the salesItem
                                DataRow newRow = dsCurrentPackingData.Tables[0].NewRow();

                                //get iteminfo
                                PmxItemInfo itemInfo = GetItemInfo(itemCode, itemInfos, sboProviderService);

                                newRow[PackingDataCustomDataSet.Columns.ITEMCODE.Name] = itemCode;
                                newRow[PackingDataCustomDataSet.Columns.ITEM_DESCRIPTION.Name] = itemInfo.Description;
                                newRow[PackingDataCustomDataSet.Columns.BARCODE.Name] = itemInfo.CodeBars;
                                newRow[PackingDataCustomDataSet.Columns.INVENTORY_UOM.Name] = itemInfo.InventoryUom;
                                newRow[PackingDataCustomDataSet.Columns.QUANTITY.Name] = FloorForDecimals(maxQty, numberOfDecimals);
                                newRow[PackingDataCustomDataSet.Columns.UOM.Name] = itemInfo.SalesUom;
                                newRow[PackingDataCustomDataSet.Columns.QUANTITY_PER_UOM.Name] = itemInfo.NumberOfItemsPerSalesUnit;
                                newRow[PackingDataCustomDataSet.Columns.PICKLIST_DOC_ENTRY.Name] = pickListDocEntry;
                                newRow[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name] = 0;
                                newRow[PackingDataCustomDataSet.Columns.SELECTED.Name] = 'N';
                                newRow[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name] = numberOfDecimals;

                                dsCurrentPackingData.Tables[0].Rows.Add(newRow);


                            }
                        }
                    }

                    if (this.GetNumberOfRows(dsCurrentPackingData) > 0)
                    {

                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Packing.IPackingMainCustomScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "TitleKey", "MSG_TITLE_SELECT_ITEMS_TO_PACK",
                                "DSPackingData", dsCurrentPackingData,
                                "PickPackRemarks", pickPackRemarks,
                                "ShipToAddress", shipToAddress,
                                "CustomerName", cardName,
                                "SSCC", sscc,
                                "PickListDocNumber", pickListDocEntry.Value
                                ));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".AddItems"))
                        {
                            DataSet dsReturnedPackingData = ExtractParameter<DataSet>(msg.Parameters, "dsPackingData");


                            Collection<PackingItemInfo> packingItemInfos = new Collection<PackingItemInfo>();
                            if (this.GetNumberOfRows(dsReturnedPackingData) > 0)
                            {
                                foreach (DataRow row in dsReturnedPackingData.Tables[0].Select(PackingDataCustomDataSet.Columns.SELECTED.Name + " = '" + SboBooleanColumnValues.Y.VALUE + "'"))
                                {

                                    string itemCode = row[PackingDataCustomDataSet.Columns.ITEMCODE.Name] as string;
                                    double quantity = Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name]);

                                    if (quantity > 0)
                                    {
                                        //we need to check if the item is a sales item
                                        if (listOfItems[itemCode].Count > 0)
                                        {
                                            //sales item
                                            //we need to pack all the ingredients
                                            foreach (string itemCodeIngredient in listOfItems[itemCode].Keys)
                                            {
                                                double qtyToConsume = quantity * listOfItems[itemCode][itemCodeIngredient];
                                                //loop tru all items
                                                foreach (DataRow rowIngredients in dsPackingData.Tables[0].Select(PackingDataCustomDataSet.Columns.ITEMCODE.Name + " = '" + itemCodeIngredient + "'"))
                                                {
                                                    int numberOfDecimals = GetItemInfo(itemCodeIngredient, itemInfos, sboProviderService).UomDecimals;
                                                    //the quantity to pack is not set, so we copy it from the quantity on line
                                                    rowIngredients[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name] = rowIngredients[PackingDataCustomDataSet.Columns.QUANTITY.Name];
                                                    //copy the items into the new ds
                                                    PackingItemInfo packingInfo = AddPackingItemInfo(rowIngredients, qtyToConsume, true, numberOfDecimals);

                                                    if (packingInfo.Quantity > 0)
                                                    {
                                                        qtyToConsume -= packingInfo.Quantity;
                                                        //reduce the quantity, so we don't use it twice
                                                        rowIngredients[PackingDataCustomDataSet.Columns.QUANTITY.Name] = Convert.ToDouble(rowIngredients[PackingDataCustomDataSet.Columns.QUANTITY.Name]) - packingInfo.Quantity;
                                                        packingItemInfos.Add(packingInfo);
                                                    }

                                                    if (qtyToConsume <= 0)
                                                    {
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                        else
                                        {
                                            //not a sales item
                                            PackingItemInfo packingInfo = AddPackingItemInfo(row, Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name]), true, Convert.ToInt32(row[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name]));

                                            if (packingInfo.Quantity > 0)
                                            {
                                                packingItemInfos.Add(packingInfo);
                                            }
                                        }
                                    }
                                }
                            }

                            if (packingItemInfos.Count > 0)
                            {
                                //ask the user to select his name
                                //get all users
                                query = BuildQuery.GetUsers();
                                DataSet dsUsers = sboProviderService.RunView(false, null, null, query);

                                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectUserScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", initialErrorKey,
                                       "TitleKey", "MSG_TITLE_SELECT_USER",
                                       "UserDS", dsUsers));

                                msg = WaitForMessage();

                                if (msg.Name.EndsWith(".UserSelected"))
                                {
                                    userSign = ExtractParameter<int>(msg.Parameters, "userSign");
                                }
                                else
                                {
                                    goto Step_PackItems;
                                }



                                //pack items
                                sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null, new DelegateWithPmxDbConnection<int?>(ExecutePackItems), packingItemInfos, luid, packingLineCode, userSign);

                                //there items are packed
                                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", initialErrorKey,
                                       "MessageKey", "MSG_ITEMS_ARE_PACKED",
                                       "ShowButton", true));

                                msg = WaitForMessage();

                            }
                        }

                        goto Step_SelectPackingTask;
                    }

                }

                //there was no packing data
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                       "InitialErrorKey", initialErrorKey,
                       "MessageKey", "MSG_ERROR_NO_PACKING_DATA_FOUND",
                       "ShowButton", true));

                msg = WaitForMessage();

                goto Step_SelectPackingTask;

            }
            catch (ProdumexException ex)
            {
                PmxError = ex.Error;
                goto Step_ShowError;
            }


            #endregion

        Step_PrintPalletLabel:

            #region PRINT PALLET LABEL

            //get the number of label copies
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
            "InitialErrorKey", initialErrorKey,
            "TitleKey", "MSG_TITLE_ENTER_NUMBER_OF_LABELS",
            "InitialQuantity", 1,
            "MaximumQuantity", -1,
            "NumberOfDigitsAfterDecimal", 0
            ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                int numberOfCopies = Convert.ToInt32(ExtractParameter<double>(msg.Parameters, "quantity"));
                //get the luid
                query = BuildQuery.GetPickListData(pickListDocEntry.Value);
                dsPickList = sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(dsPickList) > 0)
                {
                    //print the pallet label
                    try
                    {
                        sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintPalletLabel), pickListDocEntry, deviceID, packingLineCode, true, numberOfCopies);

                    }
                    catch (Produmex.Foundation.ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", ex.Error,
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }



                    goto Step_ShowDocumentsPrinted;
                }

            }

            goto Step_SelectPackingTask;

            #endregion

        Step_PrintManualItemLabel:

            #region PRINT MANUAL ITEM LABEL

            string itemCodeToPrint = null;

            goto Step_SelectCustomer;

        Step_SelectCustomer:

            #region SELECT CUSTOMER

            //get the customer
            query = BuildQuery.GetCustomers();
            DataSet dsCustomers = sboProviderService.RunView(false, null, null, query); ;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectCustomerScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_CUSTOMER",
                "CustomerDS", dsCustomers
                ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".CustomerSelected"))
            {
                cardCode = ExtractParameter<string>(msg.Parameters, "customerCode");
                goto Step_SelectItem;
            }

            goto Step_SelectPackingTask;

            #endregion

        Step_SelectItem:

            #region SELECT ITEM

            //get the customer
            query = BuildQuery.GetAllActiveItems(sboProviderService.GetDbTool());
            DataSet dsItems = sboProviderService.RunView(false, null, null, query); ;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_PRODUCT",
                "ProductDS", dsItems
                ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".ProductSelected"))
            {
                itemCodeToPrint = ExtractParameter<string>(msg.Parameters, "itemCode");

                //enter number of labels
                goto Step_EnterNumberOfLabels;
            }

            goto Step_SelectPackingTask;


            #endregion

        Step_EnterNumberOfLabels:

            #region ENTER NUMBER OF LABELS

            numberOfLabelsToPrint = 0;
                        
            //get the number of label copies
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
            "InitialErrorKey", initialErrorKey,
            "TitleKey", "MSG_TITLE_ENTER_NUMBER_OF_LABELS",
            "InitialQuantity", 1,
            "MaximumQuantity", -1,
            "NumberOfDigitsAfterDecimal", 0
            ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                numberOfLabelsToPrint = Convert.ToInt32(ExtractParameter<double>(msg.Parameters, "quantity"));
                goto Step_PrintManualItemLabels;
            }

            goto Step_SelectPackingTask;

            #endregion


        Step_PrintManualItemLabels:

            #region PRINT MANUAL ITEM LABELS

            query = BuildQuery.GetCardData(cardCode);
            DataSet dsCardData = sboProviderService.RunView(false, null, null, query);
            
            reportKey = null;

            if (GetNumberOfRows(dsCardData) > 0)
            {
                int tempReportKey;
                //all the items have the same report and cardcode!
                if (int.TryParse(Convert.ToString(dsCardData.Tables[0].Rows[0][PmxBusinessPartnerTableLino.Columns.ItemLabelReport.NAME]), out tempReportKey))
                {
                    reportKey = tempReportKey;
                }
            }

            if (reportKey.HasValue)
            {
                //print the list of items
                int numberOfCopies = Convert.ToInt32(numberOfLabelsToPrint);

                //we need to print the report
                try
                {
                    sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintItemLabels), itemCodeToPrint, cardCode, deviceID, packingLineCode, numberOfLabelsToPrint, reportKey, false);
                    goto Step_ShowDocumentsPrinted;
                }
                catch (Produmex.Foundation.ProdumexException ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                            "MessageAddendum", ex.Message,
                            "PmxError", ex.Error,
                            "ShowButton", true));

                    msg = WaitForMessage();
                }
                catch (Exception ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                            "MessageAddendum", ex.Message,
                            "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                            "ShowButton", true));

                    msg = WaitForMessage();
                }
            }

            goto Step_SelectPackingTask;

            #endregion


            #endregion

        Step_StopPickList:

            #region STOP PICK LIST

            query = BuildQuery.GetPickListData(pickListDocEntry.Value);
            DataSet dsPickListData = sboProviderService.RunView(false, null, null, query);

            try
            {


                if (this.GetNumberOfRows(dsPickListData) > 0)
                {
                    DataRow row = dsPickListData.Tables[0].Rows[0];

                    cardCode = row[PmxPickListTable.Columns.CardCode.NAME] as string;
                    cardName = row[PmxPickListTable.Columns.CardName.NAME] as string;
                    shipToAddress = row[PmxPickListTable.Columns.ShipToAddress.NAME] as string;
                    pickPackRemarks = row[PmxPickListTable.Columns.PickPackRemarks.NAME] as string;
                    sscc = row[PmxLogisticUnitIDTable.Columns.SSCC.NAME] as string;
                    int tempLUID;
                    if (int.TryParse(Convert.ToString(row[PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME]), out tempLUID))
                    {
                        luid = tempLUID;
                    }
                    else
                    {
                        luid = null;
                    }
                }


                //get the list of items to pack
                query = BuildQuery.GetPackingData(pickListDocEntry.Value, packingLineCode, true, sboProviderService.GetDbTool());
                dsPackingData = sboProviderService.RunView(false, null, null, query);

                if (GetNumberOfRows(dsPackingData) > 0)
                {

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Packing.IPackingMainCustomScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "TitleKey", "MSG_TITLE_PACKING_REMAINING",
                            "DSPackingData", dsPackingData,
                            "PickPackRemarks", pickPackRemarks,
                            "ShipToAddress", shipToAddress,
                            "CustomerName", cardName,
                            "SSCC", sscc,
                            "IsForFinish", true,
                            "PickListDocNumber", pickListDocEntry.Value
                            ));

                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Finish"))
                    {
                        DataSet dsReturnedPackingData = ExtractParameter<DataSet>(msg.Parameters, "dsPackingData");

                        //Build list of waste and unpacked items
                        Collection<PackingItemInfo> packingItemInfosWaste = new Collection<PackingItemInfo>();
                        Collection<PackingItemInfo> packingItemInfosunpacked = new Collection<PackingItemInfo>();
                        if (this.GetNumberOfRows(dsReturnedPackingData) > 0)
                        {
                            foreach (DataRow row in dsReturnedPackingData.Tables[0].Rows)
                            {
                                double quantityRemaining = Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name]);
                                double quantityUnpacked = Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY.Name]);

                                if (quantityRemaining > 0)
                                {
                                    //add unpacked qty
                                    PackingItemInfo packingInfo = AddPackingItemInfo(row, quantityRemaining, true, Convert.ToInt32(row[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name]));
                                    if (packingInfo.Quantity > 0)
                                    {
                                        packingItemInfosunpacked.Add(packingInfo);
                                    }
                                }

                                double quantityWaste = quantityUnpacked - quantityRemaining;
                                if (quantityWaste > 0)
                                {
                                    //add the waste
                                    PackingItemInfo packingInfo = AddPackingItemInfo(row, quantityWaste, false, Convert.ToInt32(row[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name]));
                                    if (packingInfo.Quantity > 0)
                                    {
                                        packingItemInfosWaste.Add(packingInfo);
                                    }
                                }
                            }
                        }

                        //add to db
                        try
                        {
                            sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null, new DelegateWithPmxDbConnection<object>(ExecuteManageUnpackedWasteItem), packingItemInfosunpacked, packingItemInfosWaste, packingLineCode);
                        }
                        catch (Exception ex)
                        {
                            initialErrorKey = ex.Message;

                            goto Step_StopPickList;
                        }

                        goto Step_ShipPickList;

                    }
                }

                goto Step_ShipPickList;

            }
            catch (ProdumexException ex)
            {
                PmxError = ex.Error;
                goto Step_ShowError;
            }



        Step_ShipPickList:

            #region SHIP PICK LIST

            try
            {

                //check if the picklist can be shipped
                listOfSalesDeliveryDocEntries = sboProviderService.InvokeMethodWithDbConnection<DocumentRef[]>(false, null, null, new DelegateWithSboConnection<DocumentRef[]>(ExecuteShipPickListWhenNeeded), pickListDocEntry.Value);

                if (listOfSalesDeliveryDocEntries != null &&
                    listOfSalesDeliveryDocEntries.Length > 0)
                {
                    //print the documents
                    goto Step_PrintDeliveryDocuments;
                }
            }
            catch (Exception ex)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                    "MessageAddendum", ex.Message,
                    "ShowButton", true));

                msg = WaitForMessage();
            }


            goto Step_SelectPackingTask;

            #endregion

            #endregion

        Step_PackMore:

            #region PACK MORE

            //does the picklist exist?
            query = BuildQuery.GetPickListData(pickListDocEntry.Value);
            dsPickList = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsPickList) > 0)
            {
                DataRow row = dsPickList.Tables[0].Rows[0];

                cardCode = row[PmxPickListTable.Columns.CardCode.NAME] as string;
                cardName = row[PmxPickListTable.Columns.CardName.NAME] as string;
                shipToAddress = row[PmxPickListTable.Columns.ShipToAddress.NAME] as string;
                pickPackRemarks = row[PmxPickListTable.Columns.PickPackRemarks.NAME] as string;
                sscc = row[PmxLogisticUnitIDTable.Columns.SSCC.NAME] as string;
                int tempLUID;
                if (int.TryParse(Convert.ToString( row[PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME]) , out tempLUID))
                {
                    luid = tempLUID;
                }
                else
                {
                    luid = null;
                }

            }




            userSign = null;

            //get the list of items to pack
            query = BuildQuery.GetItemsToPack(pickListDocEntry.Value, true);
            DataSet dsListOfItemsToPackMore = sboProviderService.RunView(false, null, null, query);

            //get the list of items to pack
            query = BuildQuery.GetPackingDataForPackingMore(pickListDocEntry.Value, packingLineCode, false, sboProviderService.GetDbTool());
            DataSet dsPackingDataMore = sboProviderService.RunView(false, null, null, query);

            try
            {


                //build list of 'normal' items, and items that need to be assembled
                Dictionary<string, Dictionary<string, double>> listOfItems = new Dictionary<string, Dictionary<string, double>>();

                foreach (DataRow row in dsListOfItemsToPackMore.Tables[0].Rows)
                {
                    string itemCode = row[PmxPickListLineTable.Columns.ItemCode.NAME] as string;
                    string salesItemItemCode = row[Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE] as string;

                    if (!string.IsNullOrEmpty(salesItemItemCode))
                    {
                        if (!listOfItems.ContainsKey(salesItemItemCode))
                        {
                            listOfItems.Add(salesItemItemCode, new Dictionary<string, double>());
                        }
                        //the item is part of a sales item, so we need to set the ingredients
                        if (!listOfItems[salesItemItemCode].ContainsKey(itemCode))
                        {
                            //set the quantity of the BOM
                            double qtyOnSalesItem = Convert.ToDouble(row[SboSalesOrderLineTable.Columns.Quantity.NAME]);
                            double componentQty = Convert.ToDouble(row[PmxComponentSalesItemViewDefinition.Columns.ComponentQuantity.NAME]);
                            listOfItems[salesItemItemCode].Add(itemCode, componentQty / qtyOnSalesItem);
                        }
                    }
                    else
                    {
                        if (!listOfItems.ContainsKey(itemCode))
                        {
                            listOfItems.Add(itemCode, new Dictionary<string, double>());
                        }
                    }
                }

                Dictionary<string, double> listOfUsedIngredients = new Dictionary<string, double>();

                //we need to make the new dataset to pass to the screen
                DataSet dsCurrentPackingData = dsPackingDataMore.Clone();
                if (this.GetNumberOfRows(dsPackingDataMore) > 0)
                {
                    foreach (string itemCode in listOfItems.Keys)
                    {
                        if (listOfItems[itemCode].Count == 0)
                        {
                            //The item is not a sales item, so we can copy the data
                            foreach (DataRow row in dsPackingDataMore.Tables[0].Select(PackingDataCustomDataSet.Columns.ITEMCODE.Name + " = '" + itemCode + "'"))
                            {
                                //copy the items into the new ds
                                dsCurrentPackingData.Tables[0].Rows.Add(row.ItemArray);
                            }
                        }
                        else
                        {
                            //the item is a sales item, so we need to add the sales item instead of the component
                            //calculate the max qty that can be packed
                            double maxQty = double.MaxValue;

                            DataRow[] rowsNumberOfDecimals = dsListOfItemsToPackMore.Tables[0].Select(Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE + " = '" + itemCode + "'");
                            int numberOfDecimals = 6;
                            if (rowsNumberOfDecimals.Length > 0)
                            {
                                numberOfDecimals = Convert.ToInt32(rowsNumberOfDecimals[0][PmxItemTable.Columns.UomDecimals.NAME]);
                            }

                            foreach (string itemCodeIngredient in listOfItems[itemCode].Keys)
                            {
                                double qtyIngredient = 0;
                                foreach (DataRow row in dsPackingDataMore.Tables[0].Select(PackingDataCustomDataSet.Columns.ITEMCODE.Name + " = '" + itemCodeIngredient + "'"))
                                {
                                    //get the qty
                                    qtyIngredient += Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY.Name]);

                                    //only keep lowest possible number of decimal
                                    int tempDecimals = Convert.ToInt32(row[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name]);
                                    if (tempDecimals < numberOfDecimals)
                                    {
                                        numberOfDecimals = tempDecimals;
                                    }

                                }
                                //reduce the qty of the ingredient with the quantity already used
                                if (listOfUsedIngredients.ContainsKey(itemCodeIngredient))
                                {
                                    qtyIngredient -= listOfUsedIngredients[itemCodeIngredient];
                                }

                                maxQty = Math.Min(maxQty, qtyIngredient / listOfItems[itemCode][itemCodeIngredient]);


                            }

                            //we need to add all the quantities we consume of the ingredient to the list
                            foreach (string itemCodeIngredient in listOfItems[itemCode].Keys)
                            {
                                if (!listOfUsedIngredients.ContainsKey(itemCodeIngredient))
                                {
                                    listOfUsedIngredients.Add(itemCodeIngredient, 0);
                                }
                                listOfUsedIngredients[itemCodeIngredient] += (listOfItems[itemCode][itemCodeIngredient] * maxQty);
                            }

                            //if (maxQty > 0)
                            {
                                //we can add a row for the salesItem
                                DataRow newRow = dsCurrentPackingData.Tables[0].NewRow();

                                //get iteminfo
                                PmxItemInfo itemInfo = GetItemInfo(itemCode, itemInfos, sboProviderService);

                                newRow[PackingDataCustomDataSet.Columns.ITEMCODE.Name] = itemCode;
                                newRow[PackingDataCustomDataSet.Columns.ITEM_DESCRIPTION.Name] = itemInfo.Description;
                                newRow[PackingDataCustomDataSet.Columns.BARCODE.Name] = itemInfo.CodeBars;
                                newRow[PackingDataCustomDataSet.Columns.INVENTORY_UOM.Name] = itemInfo.InventoryUom;
                                newRow[PackingDataCustomDataSet.Columns.QUANTITY.Name] = FloorForDecimals(maxQty, numberOfDecimals);
                                newRow[PackingDataCustomDataSet.Columns.UOM.Name] = itemInfo.SalesUom;
                                newRow[PackingDataCustomDataSet.Columns.QUANTITY_PER_UOM.Name] = itemInfo.NumberOfItemsPerSalesUnit;
                                newRow[PackingDataCustomDataSet.Columns.PICKLIST_DOC_ENTRY.Name] = pickListDocEntry;
                                newRow[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name] = 0;
                                newRow[PackingDataCustomDataSet.Columns.SELECTED.Name] = 'N';
                                newRow[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name] = numberOfDecimals;

                                dsCurrentPackingData.Tables[0].Rows.Add(newRow);


                            }
                        }
                    }

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Packing.IPackingMainCustomScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "TitleKey", "MSG_TITLE_SELECT_ITEMS_TO_PACK_MORE",
                            "DSPackingData", dsCurrentPackingData,
                            "PickPackRemarks", pickPackRemarks,
                            "ShipToAddress", shipToAddress,
                            "CustomerName", cardName,
                            "SSCC", sscc,
                            "PickListDocNumber", pickListDocEntry.Value
                            ));

                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".AddItems"))
                    {
                        DataSet dsReturnedPackingData = ExtractParameter<DataSet>(msg.Parameters, "dsPackingData");


                        Collection<PackingItemInfo> packingItemInfos = new Collection<PackingItemInfo>();
                        if (this.GetNumberOfRows(dsReturnedPackingData) > 0)
                        {
                            foreach (DataRow row in dsReturnedPackingData.Tables[0].Select(PackingDataCustomDataSet.Columns.SELECTED.Name + " = '" + SboBooleanColumnValues.Y.VALUE + "'"))
                            {

                                string itemCode = row[PackingDataCustomDataSet.Columns.ITEMCODE.Name] as string;
                                double quantity = Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name]);

                                if (quantity > 0)
                                {
                                    //we need to check if the item is a sales item
                                    if (listOfItems[itemCode].Count > 0)
                                    {
                                        //sales item
                                        //we need to pack all the ingredients
                                        foreach (string itemCodeIngredient in listOfItems[itemCode].Keys)
                                        {
                                            double qtyToConsume = quantity * listOfItems[itemCode][itemCodeIngredient];
                                            //loop tru all items
                                            foreach (DataRow rowIngredients in dsPackingDataMore.Tables[0].Select(PackingDataCustomDataSet.Columns.ITEMCODE.Name + " = '" + itemCodeIngredient + "'"))
                                            {
                                                int numberOfDecimals = GetItemInfo(itemCodeIngredient, itemInfos, sboProviderService).UomDecimals;
                                                //the quantity to pack is not set, so we copy it from the quantity on line
                                                rowIngredients[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name] = qtyToConsume;
                                                //copy the items into the new ds
                                                PackingItemInfo packingInfo = AddPackingItemInfo(rowIngredients, qtyToConsume, true, numberOfDecimals);
                                                if (packingInfo.Quantity > 0)
                                                {
                                                    packingItemInfos.Add(packingInfo);
                                                }
                                                break;
                                            }
                                        }
                                    }
                                    else
                                    {
                                        //not a sales item
                                        PackingItemInfo packingInfo = AddPackingItemInfo(row, Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name]), true, Convert.ToInt32(row[PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name]));

                                        if (packingInfo.Quantity > 0)
                                        {
                                            packingItemInfos.Add(packingInfo);
                                        }
                                       
                                    }
                                }
                            }
                        }

                        if (packingItemInfos.Count > 0)
                        {
                            //ask the user to select his name
                            //get all users
                            query = BuildQuery.GetUsers();
                            DataSet dsUsers = sboProviderService.RunView(false, null, null, query);

                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectUserScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                   "InitialErrorKey", initialErrorKey,
                                   "TitleKey", "MSG_TITLE_SELECT_USER",
                                   "UserDS", dsUsers));

                            msg = WaitForMessage();

                            if (msg.Name.EndsWith(".UserSelected"))
                            {
                                userSign = ExtractParameter<int>(msg.Parameters, "userSign");
                            }
                            else
                            {
                                goto Step_PackItems;
                            }



                            //pack items
                            sboProviderService.InvokeMethodWithDbConnection<int?>(false, null, null, new DelegateWithSboConnection<int?>(ExecutePackMoreItems), packingItemInfos, luid, packingLineCode, userSign);

                            //there items are packed
                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                   "InitialErrorKey", initialErrorKey,
                                   "MessageKey", "MSG_ITEMS_ARE_PACKED",
                                   "ShowButton", true));

                            msg = WaitForMessage();

                        }
                    }

                    goto Step_SelectPackingTask;

                }

                //there was no packing data
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                       "InitialErrorKey", initialErrorKey,
                       "MessageKey", "MSG_ERROR_NO_PACKING_DATA_FOUND",
                       "ShowButton", true));

                msg = WaitForMessage();

                goto Step_SelectPackingTask;

            }
            catch (ProdumexException ex)
            {
                PmxError = ex.Error;
                goto Step_ShowError;
            }


            #endregion

        Step_ShowDocumentsPrinted:

            #region SHOW DOCUMENTS PRINTED

            //labels printed
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
                   "InitialErrorKey", initialErrorKey,
                   "MessageKey", "MSG_REPORTS_SEND_TO_PRINTERS",
                   "ShowButton", true));

            msg = this.WaitForMessage();


            goto Step_SelectPackingTask;

            #endregion

        Step_PrintDeliveryDocuments:

            #region PRINT DELIVERY DOCUMENTS

            int numberOfReports = 0;
            try
            {
                if (listOfSalesDeliveryDocEntries != null &&
                                listOfSalesDeliveryDocEntries.Length > 0)
                {
                    foreach (DocumentRef salesDeliveryDocEntry in listOfSalesDeliveryDocEntries)
                    {
                        //trigger event
                        numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null, new DelegateWithPmxDbConnection<int>(ExecutePrintEvents), PmxPrintReportEventType.ShippingSalesDeliveryNoteCreated, packingLineCode, deviceID, salesDeliveryDocEntry.DocEntry);

                    }
                    numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null, new DelegateWithPmxDbConnection<int>(ExecutePrintEvents), PmxPrintReportEventType.ShippingPicklistShipped, packingLineCode, deviceID, pickListDocEntry.Value);
                }

            }
            catch (Produmex.Foundation.ProdumexException ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", ex.Error,
                        "ShowButton", true));

                msg = WaitForMessage();
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                        "ShowButton", true));

                msg = WaitForMessage();
            }

            if (numberOfReports > 0)
            {
                goto Step_ShowDocumentsPrinted;
            }
            else
            {
                goto Step_SelectPackingTask;
            }


            #endregion

        Step_ShowError:

            #region SHOW ERROR

            //show processing screen
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                DefaultCultureInfo.Get(),
                BuildParamCollection(
                    "PmxError", PmxError,
                    "ShowButton", true));
            msg = WaitForMessage();

            goto Step_SelectPackingTask;

            #endregion


        }

        #endregion

        #region HELPERS

        /// <summary>
        /// Gets the item info.
        /// </summary>
        /// <param name="itemCode">The item code.</param>
        /// <param name="itemInfos">The item infos.</param>
        /// <param name="sboProviderService">The sbo provider service.</param>
        /// <returns></returns>
        private PmxItemInfo GetItemInfo(string itemCode, Dictionary<string, PmxItemInfo> itemInfos, ISboProviderService sboProviderService)
        {
            if (itemInfos == null)
            {
                itemInfos = new Dictionary<string, PmxItemInfo>();
            }
            if (!itemInfos.ContainsKey(itemCode))
            {
                //get iteminfo
                itemInfos.Add(itemCode, sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), itemCode));
            }

            return itemInfos[itemCode];
        }

        /// <summary>
        /// Adds the packing item info.
        /// </summary>
        /// <param name="row">The row.</param>
        /// <param name="maxQtyToAdd">The max qty to add.</param>
        /// <param name="test">if set to <c>true</c> [test].</param>
        /// <returns></returns>
        private PackingItemInfo AddPackingItemInfo(DataRow row, double maxQtyToAdd, bool usePackQuantity, int numberOfDecimals)
        {
            PackingDataCustomDataSet.ValidateMatchesDataSet(row.Table.DataSet);

            //not a sales item
            PackingItemInfo packingInfo = new PackingItemInfo();

            packingInfo.BatchNumber = row[PackingDataCustomDataSet.Columns.BATCHNUMBER.Name] as string;
            if (row[PackingDataCustomDataSet.Columns.BEST_BEFORE_DATE.Name] is DateTime)
            {
                packingInfo.BestBeforeDate = Convert.ToDateTime(row[PackingDataCustomDataSet.Columns.BEST_BEFORE_DATE.Name]);
            }
            packingInfo.PickListDocEntry = Convert.ToInt32(row[PackingDataCustomDataSet.Columns.PICKLIST_DOC_ENTRY.Name]);
            packingInfo.ItemCode = row[PackingDataCustomDataSet.Columns.ITEMCODE.Name] as string;
            packingInfo.ItemDescription = row[PackingDataCustomDataSet.Columns.ITEM_DESCRIPTION.Name] as string;
            packingInfo.LocationCode = row[PackingDataCustomDataSet.Columns.LOCATION_CODE.Name] as string;

            packingInfo.LogisticUnitIdentKey = Convertor.ConvertToInt32Nullable(row[PackingDataCustomDataSet.Columns.LUID.Name]);
            packingInfo.ItemTransactionalInfoKey = Convertor.ConvertToInt32Nullable(row[PackingDataCustomDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name]);


            packingInfo.QualityStatusCode = row[PackingDataCustomDataSet.Columns.QUALITY_STATUS_CODE.Name] as string;
            packingInfo.Uom = row[PackingDataCustomDataSet.Columns.UOM.Name] as string;
            packingInfo.QuantityPerUom = Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_PER_UOM.Name]);

            packingInfo.Quantity = Math.Round(Math.Min(maxQtyToAdd, usePackQuantity ? Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name]) : Convert.ToDouble(row[PackingDataCustomDataSet.Columns.QUANTITY.Name])), numberOfDecimals); ;

            return packingInfo;

        }

        /// <summary>
        /// Executes the print pick list tasks.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintPickListTasks(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@pickListDocEntry", (int)parameters[0]));

            return prov.PrintReport(reportParameters, Definitions.REPORT_PATH_PICKLIST_TASKS, 1, (string)parameters[1], (string)parameters[2], (bool)parameters[3]);
        }


        /// <summary>
        /// Executes the print events.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static int ExecutePrintEvents(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);
            bool dummy;
            return prov.PrintReportsForEvent((PmxPrintReportEventType) parameters[0] , (string)parameters[1], (string)parameters[2], (int)parameters[3], out dummy);
        }

        /// <summary>
        /// Executes the print pallet label.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintPalletLabel(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@pickListDocEntry", (int)parameters[0]));

            return prov.PrintReport(reportParameters, Definitions.REPORT_PATH_PALLET_LABEL, (int)parameters[4], (string)parameters[1], (string)parameters[2], (bool)parameters[3]);
        }


        /// <summary>
        /// Executes the manage unpacked waste item.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        private static object ExecuteManageUnpackedWasteItem(PmxDbConnection conn, object[] parameters)
        {
            PmxPickListProvider prov = new PmxPickListProvider(conn);

            prov.ManageUnpackedWasteItem((Collection<PackingItemInfo>)parameters[0], (Collection<PackingItemInfo>)parameters[1], (string)parameters[2]);

            return null;
        }


        /// <summary>
        /// Executes the pack items.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static int? ExecutePackItems(PmxDbConnection conn, object[] parameters)
        {
            PackingController prov = new PackingController(conn);

            return prov.PackItemsWithUser((Collection<PackingItemInfo>)parameters[0], (int?)parameters[1], (string)parameters[2], null, (int?)parameters[3], null, false).LUID;
        }


        /// <summary>
        /// Executes the pack items.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static int? ExecutePackMoreItems(PmxDbConnection conn, object[] parameters)
        {
            //PackingController prov = new PackingController(conn);
            PmxGeneralProviderLino prov = new PmxGeneralProviderLino(conn);

            return prov.PackMoreItemsWithUser((Collection<PackingItemInfo>)parameters[0], (int?)parameters[1], (string)parameters[2], null, (int?)parameters[3]);
        }



        /// <summary>
        /// Executes the ship pick list when needed.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static DocumentRef[] ExecuteShipPickListWhenNeeded(SboConnection conn, object[] parameters)
        {
            PmxSalesDeliveryNoteProvider prov = new PmxSalesDeliveryNoteProvider(conn);

            return prov.ShipPickListWhenNeeded((int)parameters[0]);
        }

        /// <summary>
        /// Executes the print customer item labels.
        /// </summary>
        /// <param name="conn">The SboConnection to use.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintItemLabels(PmxDbConnection conn, object[] parameters)
        {
            string itemCode = (string)parameters[0];
            string cardCode = (string)parameters[1];
            string deviceID = (string)parameters[2];
            string locationCode = (string)parameters[3];
            int numberOfCopies = (int)parameters[4];
            int reportKey = (int)parameters[5];

            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            // Add parameter for the report
            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@itemcode", itemCode));
            reportParameters.Add(new ReportParameter("@cardcode", cardCode));

            return prov.PrintReport(reportParameters, reportKey, numberOfCopies, deviceID, locationCode, (bool)parameters[6]);

        }


        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }

        /// <summary>
        /// Executes the move LUID.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static string ExecuteGetParentProductionLineForLocation(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProductionLineProvider prov = new PmxOseProductionLineProvider(conn);
            return prov.GetParentProductionLineForLocation((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetItemInfo((string)parameters[0]);
        }

        private double FloorForDecimals(double qty, int numberOfDecimals)
        {
            double tempQty = qty * Math.Pow(10, numberOfDecimals);

            tempQty = Math.Floor(tempQty);

            tempQty /= Math.Pow(10, numberOfDecimals);

            return tempQty;
        }


        #endregion

        #region BUILD QUERY



        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        public static class BuildQuery
        {


            /// <summary>
            /// Gets all active items.
            /// </summary>
            /// <param name="dbTool">The database tool.</param>
            /// <returns></returns>
            public static string GetAllActiveItems(DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Inventory, ProductDataSet.Columns.GTIN.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, ProductDataSet.Columns.DEFAULT_LOCATION_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, PmxItemTable.Columns.HasBestBeforeDate.NAME, ProductDataSet.Columns.HAS_BEST_BEFORE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ManBtchNum.NAME, ProductDataSet.Columns.MANBTCHNUM.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.frozenFor.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME);

                return strBuilder.ToString();
            }

            /// <summary>
            /// Command for getting all customers order by name
            /// </summary>
            /// <returns>Command for getting all customers order by name</returns>
            public static string GetCustomers()
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardCode.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(CustomerDataSet.Columns.CUSTOMER_CODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardName.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(CustomerDataSet.Columns.CUSTOMER_NAME.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardType.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBusinessPartnerTypeColumnValues.Customer.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboBusinessPartnerTable.Columns.CardName.NAME));
                return strBuilder.ToString();
            }

            public static string GetPrinters(string parentCode, string pagesizeCode, CultureInfo dbCultureInfo)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Code.NAME, PrinterDataSet.Columns.CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Name.NAME, PrinterDataSet.Columns.PRINTER_NAME.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Default.NAME);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOsePrinterTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Code.NAME);

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.ParentCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(parentCode, dbCultureInfo));

                if (!String.IsNullOrEmpty(pagesizeCode))
                {
                    strBuilder.Append(" AND ( ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.PageSizeCode.NAME);
                    strBuilder.AppendFormat(" = {0}", SqlCommandHelper.FormatAndEscapeColumnValue(pagesizeCode, dbCultureInfo));
                    strBuilder.Append(" OR ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.PageSizeCode.NAME);
                    strBuilder.Append(" IS NULL )");
                }

                strBuilder.Append(" AND (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Active.NAME);
                strBuilder.AppendFormat(" = {0}", SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, dbCultureInfo));
                strBuilder.Append(" OR ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Active.NAME);
                strBuilder.Append(" IS NULL )");

                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePrinterTable.NAME, PmxOsePrinterTable.Columns.Default.NAME);
                strBuilder.Append(" DESC ");

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the users.
            /// </summary>
            /// <returns></returns>
            public static string GetUsers()
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboUserTable.NAME, SboUserTable.Columns.UserName.NAME, UserDataSet.Columns.USER_NAME.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboUserTable.NAME, SboUserTable.Columns.InternalKey.NAME, UserDataSet.Columns.USER_SIGN.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUserTable.NAME));

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUserTable.NAME, SboUserTable.Columns.Groups.NAME);
                strBuilder.Append(" <> 99 ");


                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUserTable.NAME, SboUserTable.Columns.UserName.NAME);

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all the production lines.
            /// </summary>
            /// <param name="warehouseCode">The warehouse code.</param>
            /// <param name="locationCode">The location code.</param>
            /// <returns>
            /// The sql statement to get all the production lines
            /// </returns>
            public static string GetProductionLines(string warehouseCode, string locationCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder,PmxOseStorageLocationGeneralTable.NAME,PmxOseStorageLocationGeneralTable.Columns.Code.NAME,LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME,PmxOseProductionLineTable.Columns.InputLocationCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.OutputLocationCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.RestLocationCode.NAME);
                

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseProductionLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseProductionLineTable.NAME, PmxOseProductionLineTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);



                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                if (!string.IsNullOrEmpty(warehouseCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

                }

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.Code.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the items to print.
            /// </summary>
            /// <param name="pickListDocEntry">The pick list doc entry.</param>
            /// <returns></returns>
            public static string GetItemsToPrint(int pickListDocEntry)
            {
                const string SalesItem_SalesOrderLineTable = "SI_" + SboSalesOrderLineTable.NAME;

                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append( "SELECT " );

                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.ItemCode.NAME, PmxPickListLineTable.Columns.ItemCode.NAME );

                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPicked.NAME, PmxPickListLineTable.Columns.QtyPicked.NAME );

                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardCode.NAME, PmxPickListTable.Columns.CardCode.NAME );

                strBuilder.AppendFormat( ", ISNULL({0}, {1}) AS {2} ",
                SqlCommandHelper.EscapeTableColumnName( SalesItem_SalesOrderLineTable, PmxSalesOrderLineTableLino.Columns.PrintItemLabel.NAME ),
                SqlCommandHelper.EscapeTableColumnName( PmxSalesOrderLineTableLino.NAME, PmxSalesOrderLineTableLino.Columns.PrintItemLabel.NAME ),
                SqlCommandHelper.EscapeColumnName( PmxSalesOrderLineTableLino.Columns.PrintItemLabel.NAME ) );

                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, PmxBusinessPartnerTable.NAME, PmxBusinessPartnerTableLino.Columns.ItemLabelReport.NAME, PmxBusinessPartnerTableLino.Columns.ItemLabelReport.NAME );

                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SalesItem_SalesOrderLineTable, SboSalesOrderLineTable.Columns.ItemCode.NAME, Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE );

                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnNameWithAs( strBuilder, SalesItem_SalesOrderLineTable, SboSalesOrderLineTable.Columns.Quantity.NAME, Definitions.COLUMN_NAME_SALES_ITEM_QUANTITY );

                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnName( strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.Quantity.NAME );
                strBuilder.Append( "/ " );
                SqlCommandHelper.AddTableColumnName( strBuilder, SalesItem_SalesOrderLineTable, SboSalesOrderLineTable.Columns.Quantity.NAME );
                strBuilder.Append( " AS " );
                strBuilder.Append( SqlCommandHelper.EscapeColumnName( Definitions.COLUMN_NAME_PART_QUANTITY ) );


                strBuilder.Append( "\r\n FROM " );
                strBuilder.Append( SqlCommandHelper.EscapeTableName( PmxPickListLineTable.NAME ) );

                strBuilder.Append( "\r\n INNER JOIN " );
                strBuilder.Append( SqlCommandHelper.EscapeTableName( PmxPickListTable.NAME ) );
                strBuilder.Append( " ON " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME );

                strBuilder.Append( "\r\n INNER JOIN " );
                strBuilder.Append( SqlCommandHelper.EscapeTableName( PmxBusinessPartnerTable.NAME ) );
                strBuilder.Append( " ON " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxBusinessPartnerTable.NAME, PmxBusinessPartnerTable.Columns.CardCode.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardCode.NAME );

                strBuilder.Append( "\r\n INNER JOIN " );
                strBuilder.Append( SqlCommandHelper.EscapeTableName( PmxPickListProposalLineTable.NAME ) );
                strBuilder.Append( " ON " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME );
                strBuilder.Append( " AND " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME );
                strBuilder.Append( " AND " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME );
                strBuilder.Append( " = " );
                strBuilder.Append( SqlCommandHelper.FormatAndEscapeColumnValue( PmxPickListProposalTable.NAME, CultureInfo.InvariantCulture ) );

                strBuilder.Append( "\r\n INNER JOIN " );
                strBuilder.Append( SqlCommandHelper.EscapeTableName( PmxSalesOrderLineTable.NAME ) );
                strBuilder.Append( " ON " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.DocEntry.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME );
                strBuilder.Append( " AND " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.LineNum.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME );
                strBuilder.Append( " AND " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME );
                strBuilder.Append( " = " );
                strBuilder.Append( SqlCommandHelper.FormatAndEscapeColumnValue( ( (int)SboLinkDocumentType.SalesOrder ).ToString(), CultureInfo.InvariantCulture ) );

                strBuilder.Append( "\r\n INNER JOIN " );
                strBuilder.Append( SqlCommandHelper.EscapeTableName( PmxComponentSalesItemViewDefinition.NAME ) );
                strBuilder.Append( " ON " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.DocEntry.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.DocEntry.NAME );
                strBuilder.Append( " AND " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.ComponentLineNum.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.LineNum.NAME );

                strBuilder.AppendFormat( "\r\n LEFT JOIN {0} AS {1} ",
                    SqlCommandHelper.EscapeTableName( SboSalesOrderLineTable.NAME ),
                    SqlCommandHelper.EscapeTableName( SalesItem_SalesOrderLineTable ) );
                strBuilder.Append( " ON " );
                SqlCommandHelper.AddTableColumnName( strBuilder, SalesItem_SalesOrderLineTable, SboSalesOrderLineTable.Columns.DocEntry.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.DocEntry.NAME );
                strBuilder.Append( " AND " );
                SqlCommandHelper.AddTableColumnName( strBuilder, SalesItem_SalesOrderLineTable, SboSalesOrderLineTable.Columns.LineNum.NAME );
                strBuilder.Append( " = " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.SalesItemLineNum.NAME );


                strBuilder.Append( "\r\n WHERE " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME );
                strBuilder.Append( " = " );
                strBuilder.Append( SqlCommandHelper.FormatAndEscapeColumnValue( pickListDocEntry, CultureInfo.InvariantCulture ) );

                strBuilder.Append( " AND (" );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTableLino.Columns.PrintItemLabel.NAME );
                strBuilder.Append( " = " );
                strBuilder.Append( SqlCommandHelper.FormatAndEscapeColumnValue( Definitions.PRINT_LABEL, CultureInfo.InvariantCulture ) );
                strBuilder.Append( " OR " );
                SqlCommandHelper.AddTableColumnName( strBuilder, SalesItem_SalesOrderLineTable, SboSalesOrderLineTable.Columns.ItemCode.NAME );
                strBuilder.Append( " IS NOT NULL ) " );


                strBuilder.Append( "\r\n ORDER BY " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME );
                strBuilder.Append( ", " );
                SqlCommandHelper.AddTableColumnName( strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.ItemCode.NAME );


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the card data.
            /// </summary>
            /// <param name="cardCode">The card code.</param>
            /// <returns></returns>
            public static string GetCardData(string cardCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxBusinessPartnerTable.NAME, PmxBusinessPartnerTableLino.Columns.ItemLabelReport.NAME, PmxBusinessPartnerTableLino.Columns.ItemLabelReport.NAME);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxBusinessPartnerTable.NAME));

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxBusinessPartnerTable.NAME, PmxBusinessPartnerTable.Columns.CardCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the pick list for picking by wave.
            /// </summary>
            /// <param name="userName">Name of the user.</param>
            /// <returns></returns>
            public static string GetPickListForPickingByWave(string userName)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.InternalKey.NAME, WaveDataSet.Columns.INTERNAL_KEY.Name);

                strBuilder.Append(", CAST(");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWaveTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.InternalKey.NAME));
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(WaveDataSet.Columns.INTERNAL_KEY_AS_STRING.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.DueDate.NAME, WaveDataSet.Columns.DUE_DATE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.Description.NAME, WaveDataSet.Columns.DESCRIPTION.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.Priority.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.SameDelivery.NAME);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWaveTable.NAME));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxUserTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.LockedBy.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxUserTable.NAME, PmxUserTable.Columns.InternalKey.NAME);


                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.InternalKey.NAME));
                strBuilder.Append(" IN  ( SELECT ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.WaveKey.NAME);
                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));


                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.PickListStatus.NAME));
                strBuilder.Append(" IN (");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.Picked.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyPacked.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(" ))");

                if (!string.IsNullOrEmpty(userName))
                {
                    strBuilder.Append(" AND (");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxWaveTable.NAME, PmxWaveTable.Columns.LockedBy.NAME);
                    strBuilder.Append(" IS NULL ");
                    strBuilder.Append(" OR ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxUserTable.NAME, PmxUserTable.Columns.UserCode.NAME);
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(userName, CultureInfo.InvariantCulture));
                    strBuilder.Append(") ");
                }


                strBuilder.Append(" ORDER BY ");
                if (!string.IsNullOrEmpty(userName))
                {
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.LockedBy.NAME));
                    strBuilder.Append(" DESC, ");
                }
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.Priority.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.DueDate.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.InternalKey.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxWaveTable.Columns.Description.NAME));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the pick list for packing.
            /// </summary>
            /// <returns></returns>
            public static string GetPickListForPacking(bool forStopPickList)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME, PickListDataSet.Columns.DOC_ENTRY.Name);

                strBuilder.Append(", CAST(");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.DocEntry.NAME));
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append(PickListDataSet.Columns.DOC_ENTRY_AS_STRING.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DueDate.NAME, PickListDataSet.Columns.DUE_DATE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardName.NAME, PickListDataSet.Columns.CUSTOMER_NAME.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.ShipToAddress.NAME, PickListDataSet.Columns.CUSTOMER_DELIVERY_ADDRESS.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardCode.NAME, PickListDataSet.Columns.CUSTOMER_CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DestStorLocCode.NAME, PickListDataSet.Columns.DEST_STORLOC_CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.Priority.NAME);
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxPickListTable.NAME, PmxPickListTable.Columns.RouteKey.NAME, PmxPickListTable.Columns.RouteKey.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxPickListTable.NAME, PmxPickListTable.Columns.PickPackRemarks.NAME, PmxPickListTable.Columns.PickPackRemarks.NAME));
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.PickPackRemarks.NAME, PickListDataSet.Columns.PICK_PACK_REMARKS.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));

                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.PickListStatus.NAME));
                strBuilder.Append(" IN (");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.Picked.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(" , ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyPacked.VALUE, CultureInfo.InvariantCulture));

                if (forStopPickList)
                {
                    strBuilder.Append(" , ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.Packed.VALUE, CultureInfo.InvariantCulture));
                    strBuilder.Append(" , ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyDelivered.VALUE, CultureInfo.InvariantCulture));

                }

                strBuilder.Append(" )");

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.Priority.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.DueDate.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.CardCode.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.ShipToAddress.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.DocEntry.NAME));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all the packing lines.
            /// </summary>
            /// <param name="warehouseCode">The warehouse code.</param>
            /// <param name="locationCode">The location code.</param>
            /// <returns>
            /// The sql statement to get all the packing lines
            /// </returns>
            public static string GetPackingLines(string warehouseCode, string locationCode)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOsePackingLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOsePackingLineTable.NAME, PmxOsePackingLineTable.Columns.Code.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);



                strBuilder.Append(" WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                if (!string.IsNullOrEmpty(warehouseCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

                }

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.Code.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.Name.NAME));


                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the items to pack.
            /// </summary>
            /// <param name="pickListDocEntry">The pick list doc entry.</param>
            /// <param name="allItems">if set to <c>true</c> [all items].</param>
            /// <returns></returns>
            public static string GetItemsToPack(int pickListDocEntry, bool allItems)
            {
                //SELECT     TOP (100) PERCENT dbo.PMX_PLLI.LineNum, dbo.PMX_PLLI.ItemCode, (dbo.PMX_PLLI.QtyPicked - dbo.PMX_PLLI.QtyPacked) AS QtyPicked, 
                //                      dbo.PMX_COMPONENT_SALES_ITEM.SalesItemLineNum, dbo.PMX_COMPONENT_SALES_ITEM.ComponentQuantity, 
                //                      dbo.PMX_COMPONENT_SALES_ITEM.DocEntry
                //FROM         dbo.PMX_PLPL INNER JOIN
                //                      dbo.PMX_PLLI ON dbo.PMX_PLPL.LineNum = dbo.PMX_PLLI.BaseLine AND dbo.PMX_PLPL.DocEntry = dbo.PMX_PLLI.BaseEntry INNER JOIN
                //                      dbo.PMX_COMPONENT_SALES_ITEM ON dbo.PMX_PLPL.BaseEntry = dbo.PMX_COMPONENT_SALES_ITEM.DocEntry AND 
                //                      dbo.PMX_PLPL.BaseLine = dbo.PMX_COMPONENT_SALES_ITEM.ComponentLineNum
                //WHERE     (dbo.PMX_PLLI.DocEntry = 2419)
                //ORDER BY dbo.PMX_COMPONENT_SALES_ITEM.SalesItemLineNum


                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME, PmxPickListLineTable.Columns.LineNum.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.ItemCode.NAME, PmxPickListLineTable.Columns.ItemCode.NAME);

                strBuilder.Append(", (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPicked.NAME);
                strBuilder.Append(" - ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPacked.NAME);
                strBuilder.Append(") AS ");
                strBuilder.Append( SqlCommandHelper.EscapeColumnName(PmxPickListLineTable.Columns.QtyPicked.NAME));


                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.SalesItemLineNum.NAME, PmxComponentSalesItemViewDefinition.Columns.SalesItemLineNum.NAME));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.ComponentQuantity.NAME, PmxComponentSalesItemViewDefinition.Columns.ComponentQuantity.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.DocEntry.NAME, PmxComponentSalesItemViewDefinition.Columns.DocEntry.NAME);
 
                strBuilder.Append(", ");
                SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.ItemCode.NAME, Definitions.COLUMN_NAME_SALES_ITEM_ITEM_CODE);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.Quantity.NAME, SboSalesOrderLineTable.Columns.Quantity.NAME);

                strBuilder.Append(", ISNULL(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UomDecimals.NAME);
                strBuilder.Append(",0) AS ");
                strBuilder.Append( SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.UomDecimals.NAME));


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListProposalTable.NAME, CultureInfo.InvariantCulture));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxComponentSalesItemViewDefinition.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.ComponentLineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(((int)SboObjectType.SalesOrder).ToString(), CultureInfo.InvariantCulture));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.SalesItemLineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.LineNum.NAME);


                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.ItemCode.NAME);


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));

                if (!allItems)
                {
                    strBuilder.Append(" AND ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPicked.NAME);
                    strBuilder.Append(" - ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.QtyPacked.NAME);
                    strBuilder.Append(" > 0 ");
                }

                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxComponentSalesItemViewDefinition.NAME, PmxComponentSalesItemViewDefinition.Columns.SalesItemLineNum.NAME);


                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the pick list data.
            /// </summary>
            /// <param name="pickListDocEntry">The pick list doc entry.</param>
            /// <returns></returns>
            public static string GetPickListData(int pickListDocEntry)
            {

                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME, PmxPickListTable.Columns.DocEntry.NAME);

                strBuilder.Append(", CAST (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.PickPackRemarks.NAME);
                strBuilder.Append(" AS NVARCHAR) AS ");
                strBuilder.Append( SqlCommandHelper.EscapeColumnName(PmxPickListTable.Columns.PickPackRemarks.NAME));


                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardCode.NAME, PmxPickListTable.Columns.CardCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.CardName.NAME, PmxPickListTable.Columns.CardName.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.ShipToAddress.NAME, PmxPickListTable.Columns.ShipToAddress.NAME);

                strBuilder.Append(", ");
                SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);

                strBuilder.Append(", ");
                SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkPickListLastMoveTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveDocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveLineNum.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);



                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocStatus.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);
                strBuilder.Append(" DESC ");


                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the packing data.
            /// </summary>
            /// <param name="pickListDocEntry">The pick list doc entry.</param>
            /// <param name="packingLineCode">The packing line code.</param>
            /// <param name="fillInUsedQuantity">if set to <c>true</c> [fill in used quantity].</param>
            /// <param name="dbTool">The database tool.</param>
            /// <returns></returns>
            public static string GetPackingData(int pickListDocEntry, string packingLineCode, bool fillInUsedQuantity, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PackingDataCustomDataSet.Columns.BATCHNUMBER.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PackingDataCustomDataSet.Columns.BEST_BEFORE_DATE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemTransactionalInfoKey.NAME, PackingDataCustomDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME, PackingDataCustomDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemDescription.NAME, PackingDataCustomDataSet.Columns.ITEM_DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationStorageLocationCode.NAME, PackingDataCustomDataSet.Columns.LOCATION_CODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME, PackingDataCustomDataSet.Columns.LUID.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationQualityStatusCode.NAME, PackingDataCustomDataSet.Columns.QUALITY_STATUS_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME, PackingDataCustomDataSet.Columns.PICKLIST_DOC_ENTRY.Name);

                strBuilder.Append(", SUM (");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.Quantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.QUANTITY.Name));

                if (!fillInUsedQuantity)
                {
                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name);
                }
                else
                {

                    strBuilder.Append(", SUM (");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.Quantity.NAME);
                    strBuilder.Append(" ) AS ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name));

                }

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(" AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.SELECTED.Name));

                strBuilder.Append(", CAST ( Row_Number() OVER ( ORDER BY  ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(" ) AS int ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.ROW_NUMBER.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.Uom.NAME, PackingDataCustomDataSet.Columns.UOM.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.QuantityPerUom.NAME, PackingDataCustomDataSet.Columns.QUANTITY_PER_UOM.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME, PackingDataCustomDataSet.Columns.INVENTORY_UOM.Name));

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales, PackingDataCustomDataSet.Columns.BARCODE.Name);


                strBuilder.Append(", ISNULL(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UomDecimals.NAME);
                strBuilder.Append(",0) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name));


                //strBuilder.Append(", CAST (");
                //strBuilder.Append(" SUM (");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.Quantity.NAME);
                //strBuilder.Append(" ) ");
                //strBuilder.Append(" AS nvarchar(32)) + ' ' + ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.Uom.NAME);
                //strBuilder.Append(" AS ");
                //strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.QUANTITY_AND_UOM.Name));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkPickListLastMoveTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveDocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveLineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListLineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME);


                strBuilder.Append(" LEFT OUTER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));


                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationStorageLocationCode.NAME);
                strBuilder.Append(" <> ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(packingLineCode, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineStatus.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.PickListLineStatus.NAME);
                strBuilder.Append(" IN ( ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.Picked.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(",");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyPacked.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(") ");


                strBuilder.Append(" GROUP BY ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemTransactionalInfoKey.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemDescription.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationStorageLocationCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationQualityStatusCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.Uom.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.QuantityPerUom.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.NumberOfItemsPerSalesUnit.NAME);

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnitToGroupBy(strBuilder, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UomDecimals.NAME);

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the packing data. This gets all the packed/unpacked lines, because the user needs to select the product to add
            /// </summary>
            /// <param name="pickListDocEntry">The pick list doc entry.</param>
            /// <param name="packingLineCode">The packing line code.</param>
            /// <param name="fillInUsedQuantity">if set to <c>true</c> [fill in used quantity].</param>
            /// <param name="dbTool">The database tool.</param>
            /// <returns></returns>
            public static string GetPackingDataForPackingMore(int pickListDocEntry, string packingLineCode, bool fillInUsedQuantity, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT ");

                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PackingDataCustomDataSet.Columns.BATCHNUMBER.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PackingDataCustomDataSet.Columns.BEST_BEFORE_DATE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemTransactionalInfoKey.NAME, PackingDataCustomDataSet.Columns.ITEMTRANSACTIONALINFOKEY.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME, PackingDataCustomDataSet.Columns.ITEMCODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemDescription.NAME, PackingDataCustomDataSet.Columns.ITEM_DESCRIPTION.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationStorageLocationCode.NAME, PackingDataCustomDataSet.Columns.LOCATION_CODE.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME, PackingDataCustomDataSet.Columns.LUID.Name));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationQualityStatusCode.NAME, PackingDataCustomDataSet.Columns.QUALITY_STATUS_CODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME, PackingDataCustomDataSet.Columns.PICKLIST_DOC_ENTRY.Name);

                strBuilder.Append(", -1.0 AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.QUANTITY.Name));

                if (!fillInUsedQuantity)
                {
                    strBuilder.Append(", ");
                    SqlCommandHelper.AddDoubleZeroColumnWithName(strBuilder, PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name);
                }
                else
                {

                    strBuilder.Append(", SUM (");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.Quantity.NAME);
                    strBuilder.Append(" ) AS ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.QUANTITY_TO_PACK.Name));

                }

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
                strBuilder.Append(" AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.SELECTED.Name));

                strBuilder.Append(", CAST ( Row_Number() OVER ( ORDER BY  ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(" ) AS int ) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.ROW_NUMBER.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.Uom.NAME, PackingDataCustomDataSet.Columns.UOM.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.QuantityPerUom.NAME, PackingDataCustomDataSet.Columns.QUANTITY_PER_UOM.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME, PackingDataCustomDataSet.Columns.INVENTORY_UOM.Name));

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales, PackingDataCustomDataSet.Columns.BARCODE.Name);


                strBuilder.Append(", ISNULL(");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UomDecimals.NAME);
                strBuilder.Append(",0) AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.NUMBER_OF_DECIMALS_UOM.Name));


                //strBuilder.Append(", CAST (");
                //strBuilder.Append(" SUM (");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.Quantity.NAME);
                //strBuilder.Append(" ) ");
                //strBuilder.Append(" AS nvarchar(32)) + ' ' + ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.Uom.NAME);
                //strBuilder.Append(" AS ");
                //strBuilder.Append(SqlCommandHelper.EscapeColumnName(PackingDataCustomDataSet.Columns.QUANTITY_AND_UOM.Name));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkPickListLastMoveTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveDocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveLineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME);

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);


                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListLineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME);


                strBuilder.Append(" LEFT OUTER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);

                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));


                //strBuilder.Append(" AND ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationStorageLocationCode.NAME);
                //strBuilder.Append(" <> ");
                //strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(packingLineCode, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineStatus.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                //strBuilder.Append(" AND ");
                //SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.PickListLineStatus.NAME);
                //strBuilder.Append(" IN ( ");
                //strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.Picked.VALUE, CultureInfo.InvariantCulture));
                //strBuilder.Append(",");
                //strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(PmxPickListStatusColumnValues.PartiallyPacked.VALUE, CultureInfo.InvariantCulture));
                //strBuilder.Append(") ");


                strBuilder.Append(" GROUP BY ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemTransactionalInfoKey.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.ItemDescription.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationStorageLocationCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationQualityStatusCode.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.Uom.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.QuantityPerUom.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.NumberOfItemsPerSalesUnit.NAME);

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnitToGroupBy(strBuilder, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UomDecimals.NAME);

                return strBuilder.ToString();
            }
        }

        #endregion
    }
}
